# -*- coding: utf-8 -*-
"""
jax_fix use for provide information to help fix problem
"""
import sys


def main():
    """
    Main function
    Returns:

    """
    info_dict = {
        "pip": "Upgrade pip: python -m pip install --upgrade pip",
    }
    if len(sys.argv) == 1:
        print("Usage: jax-fix <command>")
        print("arguments:")
        for key in info_dict.keys():
            print(key)
        exit(0)
    if sys.argv[1] in info_dict.keys():
        print(info_dict[sys.argv[1]])
    else:
        print("Usage: jax-fix <command>")
        print("arguments:")
        for key in info_dict.keys():
            print(key)



import platform
import psutil

def get_process_info():
    processes = []
    for process in psutil.process_iter(['pid', 'name', 'cpu_percent', 'memory_percent']):
        processes.append(process.info)
    return processes

def print_process_info():
    processes = get_process_info()
    filtered_processes = [process for process in processes if process['cpu_percent'] is not None]
    sorted_processes = sorted(filtered_processes, key=lambda x: x['cpu_percent'], reverse=True)
    print("PID\tCPU%\tMemory%\tName")
    for process in sorted_processes:
        print(f"{process['pid']}\t{process['cpu_percent']}\t{process['memory_percent']}\t{process['name']}")

def main():
    system = platform.system()
    if system == 'Linux' or system == 'Darwin':
        print_process_info()
    else:
        print("Unsupported operating system.")

if __name__ == '__main__':
    main()



"""
nd command
"""
import sys
from jax_tools.network_test import check_connectivity


def main():
    """
    Main function
    Returns:

    """
    if len(sys.argv) == 3:
        target_ip = sys.argv[1]
        target_port = int(sys.argv[2])
        check_connectivity(target_ip, target_port, True)
    else:
        print("Usage: nd <TargetIP> <TargetPort>")



# -*- coding:utf-8 -*-
"""
pypi定义打包信息的文件
"""
import setuptools


def read_file(file_path):
    """
    Read file
    Args:
        file_path (str): file path

    Returns:

    """
    with open(file_path, 'r') as f:
        return f.read()


setuptools.setup(
    name="jax-tools",
    version="1.0.85",
    author=u"Jax",
    author_email='alvin.wan.cn@hotmail.com',
    description=u"Jax common tools library",
    platforms=['CentOS', 'Redhat', 'MacOS', 'Windows'],
    long_description=read_file('README.md'),
    long_description_content_type="text/markdown",
    url="https://jax-arsenals.com",
    license="MIT Licence",
    python_requires=">=3.0.0",
    # 定义要构建到包中的文件列表，这些文件会放到/usr/local下
    data_files=[],
    packages=['jax_tools', 'jax_tools.utils', 'jax_tools.cmd', 'jax_tools.cmd_tools'],
    exclude=['build.py'],
    package_dir={},
    package_data={
        # 'jax_tools': ['*.md'],
        '': ['CHANGELOG.md']
    },
    include_package_data=True,
    keywords=['jax'],
    # 安装时需要执行的脚本列表，如可用于管理配置文件
    scripts=[],
    download_url="https://jax-arsenals.com",
    # 定义可以为哪些模块提供依赖
    provides=[],
    install_requires=[
        "ping3 >= 4.0.4",
        "paramiko >= 3.2.0",
        "colorlog >= 6.7.0",
        "pycryptodome >= 3.18.0",
        "pyperclip >= 1.8.2",
        "psutil >= 5.9.5",
    ],
    # 定义entry points, 前面的sta指的是命令，第二个sta表示模块名，也是目录名，第三个sta表示脚本名，最好那个main，表示sta.py中的main函数
    entry_points={
        'console_scripts': [
            'jax=jax_tools.cmd.jax:main',
            'nd=jax_tools.cmd.nd:main',
            'jax-nd=jax_tools.cmd.nd:main',
            'jax-encrypt=jax_tools.cmd.jax_encrypt:main',
            'jax-fix=jax_tools.cmd.jax_fix:main',
            'jax-pam=jax_tools.cmd.jax_pam:main',
            'jax-gpt=jax_tools.cmd.jax_gpt:main',
        ]
    }
)



# -*- coding: utf-8 -*-
"""
AES encryption method
"""
import string
import secrets
import base64
import hmac
import hashlib
import os
from Crypto.Cipher import AES
from base64 import b64encode
from base64 import b64decode
from Crypto.Util.Padding import pad, unpad
from jax_tools.utils import settings as st


def get_local_key():
    """
    Ensure that the encryption key exists locally, if not, generate a new one
    Returns:

    """
    if os.path.exists(st.JAX_KEY_FILE):
        encryption_key = open(st.JAX_KEY_FILE, 'r').read().strip()
        if len(encryption_key) != 32:
            msg = 'encryption key length must be 32, please check file {} correct, or delete this file and try again.' \
                .format(st.JAX_KEY_FILE)
            print(msg)
            exit(1)
    else:
        # generate encryption key
        encryption_key = ''.join(secrets.choice(string.ascii_letters + string.digits) for _ in range(32))
        if os.path.exists(st.JAX_DATA_DIR) is False:
            os.mkdir(st.JAX_DATA_DIR)
        with open(st.JAX_KEY_FILE, 'w') as f:
            f.write(encryption_key)
    return encryption_key


class AESCipher(object):
    """
    AES Encryption Class, can use CBC or GCM mode, default is GCM mode
    You can use encrypt_body and decrypt_body to encrypt and decrypt request body
    You can use encrypt_content and decrypt_content to encrypt and decrypt content
    """

    # key for encrypting sensitive data
    def __init__(self, key=get_local_key(), cbc_mode=False):
        """
        Init method
        Args:
            key: give a token  as a key use for encrypting request body
        """
        if len(key) > 32:
            # Get the key from key 8:24
            self.key = key[8:24]
        else:
            self.key = key
        self.length = 8
        # Default encryption mode is GCM, if you want to use CBC, set it to True
        self.cbc_mode = cbc_mode
        if self.cbc_mode:
            self.encryption_mode = AES.MODE_CBC
        else:
            self.encryption_mode = AES.MODE_GCM

    @staticmethod
    def get_random_str(length):
        """
        Get a string from random length.
        Args:
            length:

        Returns:

        """
        # get secret random
        random_str = ''.join(secrets.choice(string.ascii_letters + string.digits) for _ in range(length))
        return random_str

    def encrypt(self, data, key=str()):
        """
        Encryption base  method
        Args:
            data:
            key:

        Returns:

        """
        if key == str():
            key = self.key
        key = key.encode('utf-8')
        iv = self.get_random_str(AES.block_size)
        cipher = AES.new(key, self.encryption_mode, iv.encode('utf-8'))
        cipher_text = b64encode(cipher.encrypt(pad(data.encode('utf-8'), AES.block_size))).decode()
        result = "{0}{1}{2}{3}".format(iv[:self.length], cipher_text[:self.length], iv[self.length:],
                                       cipher_text[self.length:])
        return result

    def decrypt(self, data, key=str()):
        """
        Decryption base method
        Args:
            data: Data
            key: Encryption key

        Returns:

        """
        if key == str():
            key = self.key
        key = key.encode('utf-8')
        iv = "{0}{1}".format(data[0:self.length], data[self.length
                                                       * 2:self.length * 2 + (AES.block_size - self.length)])
        cipher_text = "{0}{1}".format(
            data[self.length: self.length * 2], data[self.length + AES.block_size:])
        cipher = AES.new(key, self.encryption_mode, iv.encode('utf-8'))
        try:
            plaintext = unpad(cipher.decrypt(b64decode(cipher_text)), AES.block_size).decode()
        except ValueError:
            plaintext = 'Failed decrypt, please give a correct encrypted string'
        return plaintext


def encrypt_hmac_sha256(plaintext, secret_key):
    """
    Use HMAC-SHA256 to do irreversible encryption, such as encrypting user passwords

    Args:
        plaintext: plaintext
        secret_key: secret key

    Returns:

    """
    secret_key = secret_key.encode('utf-8')
    plaintext = plaintext.encode('utf-8')
    cipher_data = base64.b64encode(
        hmac.new(
            secret_key,
            plaintext,
            digestmod=hashlib.sha256).digest())
    return cipher_data.decode()


